/* converthp:
 *
 * Converts a hp49 object (with no additional trailing bytes) into a source
 * code readable by SASM.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define BYTES_PER_LINE	20

void
print_nibble(FILE *stream, int nibble)
{
  if (0 <= nibble && nibble <= 9)
    fputc('0' + nibble, stream);
  else if (0x0a <= nibble && nibble <= 0x0f)
    fputc('A' - 0x0a + nibble, stream);
  else
    fputc('?', stream);    
  return;
}


int
main (int argc, char **argv)
{
  FILE *ifile, *ofile;
  unsigned char buf[8];
  int n;
  int printwarning = 1;

  if (argc < 3) {
    fprintf(stderr, "Usage: %s <HP4x_file> <output.s> [-nowarning]\n", argv[0]);
    exit (EXIT_FAILURE);
  }

  if (! (ifile = fopen(argv[1], "rb"))) {
    perror(argv[1]);
    exit(EXIT_FAILURE);
  }

  if (fread(buf, 1, 4, ifile) != 4) {
    perror(argv[1]);
    exit(EXIT_FAILURE);
  }
  if (strncmp(buf, "HPHP", 4)) {
    fprintf(stderr, "%s: Not a HP4x file\n", argv[1]);
    exit(EXIT_FAILURE);
  }
  if (fread(buf, 1, 4, ifile) != 4) {
    perror(argv[1]);
    exit(EXIT_FAILURE);
  }
  buf[4] = '\0';

  if (! (ofile = fopen(argv[2], "w"))) {
    perror(argv[2]);
    exit(EXIT_FAILURE);
  }

  fprintf(ofile, "ASSEMBLE\n" "*\tNIBASC /HPHP%s/\n", buf);

  n = 0;
  while (fread(buf, 1, 1, ifile) == 1)
  {
    if (n % BYTES_PER_LINE == 0)
      fputs("\tNIBHEX ", ofile);

    print_nibble(ofile, buf[0] & 0x0f);
    print_nibble(ofile, buf[0] >> 4);

    if (++n % BYTES_PER_LINE == 0)
      fputc('\n', ofile);
  }

  if (argc >= 4) {
    if (! strcmp(argv[3], "-nowarning"))
      printwarning = 0;
  }

  if (printwarning)
    fputs("\n\n\tWARNING: You must remove the last nibble if needed.\n", ofile);

  fputs("\nRPL\n", ofile);

  fclose(ofile);
  fclose(ifile);

  exit (EXIT_SUCCESS);
}
